@@ -1,4 +1,5 @@
-"""
+""" Parallelized Association and Correlation matrix
+ 
 This module provides parallelized methods for computing associations.
 Namely, correlation, correlation ratio, Theil's U, Cramer's V
 
@@ -15,8 +16,6 @@ import scipy.stats as ss
 import matplotlib.pyplot as plt
 import scipy.cluster.hierarchy as sch
 import scipy.stats as ss
-import holoviews as hv
-import panel as pn
 
 
 from mpl_toolkits.axes_grid1 import make_axes_locatable
@@ -81,9 +80,12 @@ def weighted_conditional_entropy(x, y, sample_weight=None):
 
     if sample_weight is None:
         sample_weight = np.ones(len(x))
+    elif np.count_nonzero(sample_weight) == 0:
+        raise ValueError(
+            "All elements in sample_weight are zero. Cannot divide by zero."
+        )
 
     df = pd.DataFrame({"x": x, "y": y, "sample_weight": sample_weight})
-    # df = df.fillna(0)
     tot_weight = df["sample_weight"].sum()
     y_counter = df[["y", "sample_weight"]].groupby("y").sum().to_dict()
     y_counter = y_counter["sample_weight"]
@@ -91,9 +93,10 @@ def weighted_conditional_entropy(x, y, sample_weight=None):
     xy_counter = xy_counter["sample_weight"]
     h_xy = 0.0
     for xy in xy_counter.keys():
-        p_xy = xy_counter[xy] / tot_weight
-        p_y = y_counter[xy[1]] / tot_weight
-        h_xy += p_xy * math.log(p_y / p_xy, math.e)
+        p_xy = xy_counter[xy] / tot_weight if tot_weight != 0 else 0
+        p_y = y_counter[xy[1]] / tot_weight if tot_weight != 0 else 0
+        if p_xy != 0:
+            h_xy += p_xy * math.log(p_y / p_xy, math.e)
     return h_xy
 
 
@@ -122,7 +125,6 @@ def weighted_theils_u(x, y, sample_weight=None, as_frame=False):
         sample_weight = np.ones(len(x))
 
     df = pd.DataFrame({"x": x, "y": y, "sample_weight": sample_weight})
-    # df = df.fillna(0)
     tot_weight = df["sample_weight"].sum()
     y_counter = df[["y", "sample_weight"]].groupby("y").sum().to_dict()
     y_counter = y_counter["sample_weight"]
@@ -134,29 +136,27 @@ def weighted_theils_u(x, y, sample_weight=None, as_frame=False):
     xy_counter = xy_counter["sample_weight"]
     h_xy = 0.0
     for xy in xy_counter.keys():
-        p_xy = xy_counter[xy] / tot_weight
-        p_y = y_counter[xy[1]] / tot_weight
-        h_xy += p_xy * math.log(p_y / p_xy, math.e)
+        p_xy = xy_counter[xy] / tot_weight if tot_weight != 0 else 0
+        p_y = y_counter[xy[1]] / tot_weight if tot_weight != 0 else 0
+        if p_xy != 0:
+            h_xy += p_xy * math.log(p_y / p_xy, math.e)
 
     if h_x == 0:
         return 1.0
     else:
         u = (h_x - h_xy) / h_x
-        if -_PRECISION <= u < 0.0 or 1.0 < u <= 1.0 + _PRECISION:
-            rounded_u = 0.0 if u < 0 else 1.0
+        if abs(u) < _PRECISION or abs(u - 1.0) < _PRECISION:
+            rounded_u = round(u)
             warnings.warn(
                 f"Rounded U = {u} to {rounded_u}. This is probably due to floating point precision issues.",
                 RuntimeWarning,
             )
-            teil_u_val = rounded_u
-        else:
-            teil_u_val = u
+            u = rounded_u
+
     if as_frame:
-        return pd.DataFrame(
-            {"row": x.name, "col": y.name, "val": teil_u_val}, index=[0]
-        )
+        return pd.DataFrame({"row": x.name, "col": y.name, "val": u}, index=[0])
     else:
-        return teil_u_val
+        return u
 
 
 def theils_u_matrix(X, sample_weight=None, n_jobs=-1, handle_na="drop"):
@@ -522,7 +522,7 @@ def correlation_ratio(x, y, sample_weight=None, as_frame=False):
             for k in np.unique(y)
         ]
     else:
-        TypeError(
+        raise TypeError(
             "one of the two series should be categorical/object and the other numerical"
         )
 
@@ -1436,19 +1436,29 @@ def _callable_association_matrix_fn(
 
 
 def f_oneway_weighted(*args):
-    """f_oneway_weighted calculates the weighted F-statistic
-    (continuous target, categorical predictor)
+    """
+    Calculate the weighted F-statistic for one-way ANOVA (continuous target, categorical predictor).
+
+    Parameters
+    ----------
+    X : array-like of shape (n_samples,)
+        The predictor dataframe.
+    y : array-like of shape (n_samples,)
+        The target vector.
+    sample_weight : array-like of shape (n_samples,), optional
+        The weight vector, by default None.
 
     Returns
     -------
     float
-        value of the F-statistic
+        The value of the F-statistic.
+
+    Notes
+    -----
+    The F-statistic is calculated as:
 
-    Notes:
-    ------
-    F-statistic is calculated as:
     .. math::
-        F(rf)=\frac{\sum_i (\bar{Y}_{i \bullet}-\bar{Y})^2 \mathbin{/} (K-1)}{\sum_i \sum_k (\bar{Y}_{ij}-\bar{Y}_{i\bullet})^2 \mathbin{/} (N - K)}
+        F(rf) = \\frac{\\sum_i (\\bar{Y}_{i \\bullet} - \\bar{Y})^2 / (K-1)}{\\sum_i \\sum_k (\\bar{Y}_{ij} - \\bar{Y}_{i\\bullet})^2 / (N - K)}
     """
     # how many levels (predictor)
     n_classes = len(args)
@@ -1559,7 +1569,7 @@ def f_cat_regression_parallel(X, y, sample_weight=None, n_jobs=-1, handle_na="dr
         y.name = "target"
 
     target = y.name
-    X = pd.concat([X, y], axis=1, ignore_index=False)
+    X[target] = y.values
     # sanity checks
     X, sample_weight = _check_association_input(X, sample_weight, handle_na)
 
@@ -1583,32 +1593,31 @@ def f_cont_regression_parallel(
     X, y, sample_weight=None, n_jobs=-1, force_finite=True, handle_na="drop"
 ):
     """Univariate linear regression tests returning F-statistic.
+
     Quick linear model for testing the effect of a single regressor,
     sequentially for many regressors.
     This is done in 2 steps:
-    1. The cross correlation between each regressor and the target is computed
-       using
+    1. The cross-correlation between each regressor and the target is computed using:
            E[(X[:, i] - mean(X[:, i])) * (y - mean(y))] / (std(X[:, i]) * std(y))
-    2. It is converted to an F score ranks
-    features in the same order if all the features are positively correlated
-    with the target.
-    Note it is therefore recommended as a feature selection criterion to identify
-    potentially predictive feature for a downstream classifier, irrespective of
+    2. It is converted to an F score ranks features in the same order if all the features
+       are positively correlated with the target.
+    Note that it is therefore recommended as a feature selection criterion to identify
+    potentially predictive features for a downstream classifier, irrespective of
     the sign of the association with the target variable.
 
     Parameters
     ----------
     X : array-like of shape (n_samples, n_features)
-        predictor dataframe
+        The predictor dataframe.
     y : array-like of shape (n_samples,)
-        The target vector
+        The target vector.
     sample_weight : array-like of shape (n_samples,), optional
-        The weight vector, by default None
+        The weight vector, by default None.
     n_jobs : int, optional
-        the number of cores to use for the computation, by default -1
+        The number of cores to use for the computation, by default -1.
     handle_na : str, optional
-        either drop rows with na, fill na with 0 or do nothing, by default "drop"
-    force_finite :
+        Either drop rows with NaN, fill NaN with 0, or do nothing, by default "drop".
+    force_finite : bool, optional
         Whether or not to force the F-statistics and associated p-values to
         be finite. There are two cases where the F-statistic is expected to not
         be finite:
@@ -1617,23 +1626,22 @@ def f_cont_regression_parallel(
           `np.nan` values in the F-statistic and p-value. When
           `force_finite=True`, the F-statistic is set to `0.0` and the
           associated p-value is set to `1.0`.
-        - when the a feature in `X` is perfectly correlated (or
+        - when a feature in `X` is perfectly correlated (or
           anti-correlated) with the target `y`. In this case, the F-statistic
           is expected to be `np.inf`. When `force_finite=True`, the F-statistic
           is set to `np.finfo(dtype).max`.
 
-
     Returns
     -------
-    float :
-        F-statistic for each feature of shape (n_features,)
+    f_statistic : array-like of shape (n_features,)
+        F-statistic for each feature.
     """
     if not isinstance(y, pd.Series):
         y = pd.Series(y)
         y.name = "target"
 
     target = y.name
-    X = pd.concat([X, y], axis=1, ignore_index=False)
+    X[target] = y.values
     # sanity checks
     X, sample_weight = _check_association_input(X, sample_weight, handle_na)
 
@@ -1661,39 +1669,36 @@ def f_cont_regression_parallel(
 def f_stat_regression_parallel(
     X, y, sample_weight=None, n_jobs=-1, force_finite=True, handle_na="drop"
 ):
-    """f_stat_regression_parallel computes the weighted explained variance for the provided categorical
-    and numerical predictors using parallelization of the code.
+    """
+    Compute the weighted explained variance for the provided categorical and numerical predictors using parallelization.
 
     Parameters
     ----------
     X : array-like of shape (n_samples, n_features)
-        predictor dataframe
+        The predictor dataframe.
     y : array-like of shape (n_samples,)
-        The target vector
+        The target vector.
     sample_weight : array-like of shape (n_samples,), optional
-        The weight vector, by default None
+        The weight vector, by default None.
     n_jobs : int, optional
-        the number of cores to use for the computation, by default -1
+        The number of cores to use for the computation, by default -1.
     handle_na : str, optional
-        either drop rows with na, fill na with 0 or do nothing, by default "drop"
-    force_finite :
-        Whether or not to force the F-statistics and associated p-values to
-        be finite. There are two cases where the F-statistic is expected to not
-        be finite:
-        - when the target `y` or some features in `X` are constant. In this
-          case, the Pearson's R correlation is not defined leading to obtain
-          `np.nan` values in the F-statistic and p-value. When
-          `force_finite=True`, the F-statistic is set to `0.0` and the
-          associated p-value is set to `1.0`.
-        - when the a feature in `X` is perfectly correlated (or
-          anti-correlated) with the target `y`. In this case, the F-statistic
-          is expected to be `np.inf`. When `force_finite=True`, the F-statistic
-          is set to `np.finfo(dtype).max`.
+        Either drop rows with NA, fill NA with 0, or do nothing, by default "drop".
+    force_finite : bool, optional
+        Whether or not to force the F-statistics and associated p-values to be finite.
+        There are two cases where the F-statistic is expected to not be finite:
+        - When the target `y` or some features in `X` are constant. In this case,
+          the Pearson's R correlation is not defined leading to obtain `np.nan`
+          values in the F-statistic and p-value. When `force_finite=True`, the
+          F-statistic is set to `0.0` and the associated p-value is set to `1.0`.
+        - When a feature in `X` is perfectly correlated (or anti-correlated)
+          with the target `y`. In this case, the F-statistic is expected to be `np.inf`.
+          When `force_finite=True`, the F-statistic is set to `np.finfo(dtype).max`.
 
     Returns
     -------
     pd.Series
-        the value of the F-statistic for each predictor
+        The value of the F-statistic for each predictor.
     """
     f_stat_cont_series = f_cont_regression_parallel(
         X,
@@ -1804,7 +1809,7 @@ def f_cont_classification_parallel(
         y.name = "target"
 
     target = y.name
-    X = pd.concat([X, y], axis=1, ignore_index=False)
+    X[target] = y.values
     # sanity checks
     X, sample_weight = _check_association_input(X, sample_weight, handle_na)
 
@@ -1832,50 +1837,51 @@ def f_cat_classification_parallel(
     force_finite=True,
     handle_na="drop",
 ):
-    """Univariate information dependence
-    It is converted to an F score ranks features in the same order if
-    all the features are positively correlated with the target.
-    Note it is therefore recommended as a feature selection criterion to identify
-    potentially predictive feature for a downstream classifier, irrespective of
+    """
+    Univariate information dependence.
+
+    It ranks features in the same order if all the features are positively correlated with the target.
+    Note that it is therefore recommended as a feature selection criterion to identify
+    potentially predictive features for a downstream classifier, irrespective of
     the sign of the association with the target variable.
 
     Parameters
     ----------
     X : array-like of shape (n_samples, n_features)
-        predictor dataframe
+        The predictor dataframe.
     y : array-like of shape (n_samples,)
-        The target vector
+        The target vector.
     sample_weight : array-like of shape (n_samples,), optional
-        The weight vector, by default None
+        The weight vector, by default None.
     n_jobs : int, optional
-        the number of cores to use for the computation, by default -1
+        The number of cores to use for the computation, by default -1.
     handle_na : str, optional
-        either drop rows with na, fill na with 0 or do nothing, by default "drop"
-    force_finite :
+        Either drop rows with NaN, fill NaN with 0, or do nothing, by default "drop".
+    force_finite : bool, optional
         Whether or not to force the F-statistics and associated p-values to
         be finite. There are two cases where the F-statistic is expected to not
         be finite:
-        - when the target `y` or some features in `X` are constant. In this
-          case, the Pearson's R correlation is not defined leading to obtain
-          `np.nan` values in the F-statistic and p-value. When
-          `force_finite=True`, the F-statistic is set to `0.0` and the
-          associated p-value is set to `1.0`.
-        - when the a feature in `X` is perfectly correlated (or
-          anti-correlated) with the target `y`. In this case, the F-statistic
-          is expected to be `np.inf`. When `force_finite=True`, the F-statistic
-          is set to `np.finfo(dtype).max`.
+            - when the target `y` or some features in `X` are constant. In this
+              case, the Pearson's R correlation is not defined leading to obtain
+              `np.nan` values in the F-statistic and p-value. When
+              `force_finite=True`, the F-statistic is set to `0.0` and the
+              associated p-value is set to `1.0`.
+            - when a feature in `X` is perfectly correlated (or
+              anti-correlated) with the target `y`. In this case, the F-statistic
+              is expected to be `np.inf`. When `force_finite=True`, the F-statistic
+              is set to `np.finfo(dtype).max`.
 
     Returns
     -------
-    f_statistic :
-        F-statistic for each feature of shape (n_features,)
+    f_statistic : array-like of shape (n_features,)
+        F-statistic for each feature.
     """
     if not isinstance(y, pd.Series):
         y = pd.Series(y)
         y.name = "target"
 
     target = y.name
-    X = pd.concat([X, y], axis=1, ignore_index=False)
+    X[target] = y.values
     # sanity checks
     X, sample_weight = _check_association_input(X, sample_weight, handle_na)
 
@@ -1905,39 +1911,36 @@ def f_cat_classification_parallel(
 def f_stat_classification_parallel(
     X, y, sample_weight=None, n_jobs=-1, force_finite=True, handle_na="drop"
 ):
-    """f_stat_classification_parallel computes the weighted ANOVA F-value for the provided categorical
-    and numerical predictors using parallelization of the code.
+    """
+    Compute the weighted ANOVA F-value for the provided categorical and numerical predictors using parallelization.
 
     Parameters
     ----------
     X : array-like of shape (n_samples, n_features)
-        predictor dataframe
+        The predictor dataframe.
     y : array-like of shape (n_samples,)
-        The target vector
+        The target vector.
     sample_weight : array-like of shape (n_samples,), optional
-        The weight vector, by default None
+        The weight vector, by default None.
     n_jobs : int, optional
-        the number of cores to use for the computation, by default -1
+        The number of cores to use for the computation, by default -1.
     handle_na : str, optional
-        either drop rows with na, fill na with 0 or do nothing, by default "drop"
-    force_finite :
-        Whether or not to force the F-statistics and associated p-values to
-        be finite. There are two cases where the F-statistic is expected to not
-        be finite:
-        - when the target `y` or some features in `X` are constant. In this
-          case, the Pearson's R correlation is not defined leading to obtain
-          `np.nan` values in the F-statistic and p-value. When
-          `force_finite=True`, the F-statistic is set to `0.0` and the
-          associated p-value is set to `1.0`.
-        - when the a feature in `X` is perfectly correlated (or
-          anti-correlated) with the target `y`. In this case, the F-statistic
-          is expected to be `np.inf`. When `force_finite=True`, the F-statistic
-          is set to `np.finfo(dtype).max`.
+        Either drop rows with NA, fill NA with 0, or do nothing, by default "drop".
+    force_finite : bool, optional
+        Whether or not to force the F-statistics and associated p-values to be finite.
+        There are two cases where the F-statistic is expected to not be finite:
+        - When the target `y` or some features in `X` are constant. In this case,
+          the Pearson's R correlation is not defined leading to obtain `np.nan`
+          values in the F-statistic and p-value. When `force_finite=True`, the
+          F-statistic is set to `0.0` and the associated p-value is set to `1.0`.
+        - When a feature in `X` is perfectly correlated (or anti-correlated)
+          with the target `y`. In this case, the F-statistic is expected to be `np.inf`.
+          When `force_finite=True`, the F-statistic is set to `np.finfo(dtype).max`.
 
     Returns
     -------
     pd.Series
-        the value of the F-statistic for each predictor
+        The value of the F-statistic for each predictor.
     """
     f_stat_cont_series = f_cont_classification_parallel(
         X, y, sample_weight, n_jobs, handle_na=handle_na
@@ -2110,29 +2113,27 @@ def xy_to_matrix(xy):
 
 
 def cluster_sq_matrix(sq_matrix, method="ward"):
-    """cluster_sq_matrix applies agglomerative clustering in order to sort
-    a correlation matrix.
+    """
+    Apply agglomerative clustering to sort a square correlation matrix.
 
     Parameters
     ----------
     sq_matrix : pd.DataFrame
-        a square correlation matrix
+        A square correlation matrix.
     method : str, optional
-        linkage method, by default "ward"
+        The linkage method, by default "ward".
 
     Returns
     -------
     pd.DataFrame
-        a sorted square matrix
+        A sorted square matrix.
 
-    Example:
-    --------
-    >>> assoc = association_matrix(
-    ...     iris_df,
-    ...     plot=False
-    ... )
+    Example
+    -------
+    >>> from some_module import association_matrix, cluster_sq_matrix
+
+    >>> assoc = association_matrix(iris_df, plot=False)
     >>> assoc_clustered = cluster_sq_matrix(assoc, method="complete")
-
     """
     d = sch.distance.pdist(sq_matrix.values)
     L = sch.linkage(d, method=method)
@@ -2142,28 +2143,6 @@ def cluster_sq_matrix(sq_matrix, method="ward"):
     sq_matrix = sq_matrix.reindex(columns, axis=0)
     return sq_matrix
 
-    """
-    Create a heatmap from a numpy array and two lists of labels.
-
-    Parameters
-    ----------
-    data
-        A 2D numpy array of shape (M, N).
-    row_labels
-        A list or array of length M with the labels for the rows.
-    col_labels
-        A list or array of length N with the labels for the columns.
-    ax
-        A `matplotlib.axes.Axes` instance to which the heatmap is plotted.  If
-        not provided, use current axes or create a new one.  Optional.
-    cbar_kw
-        A dictionary with arguments to `matplotlib.Figure.colorbar`.  Optional.
-    cbarlabel
-        The label for the colorbar.  Optional.
-    **kwargs
-        All other arguments are forwarded to `imshow`.
-    """
-
 
 def heatmap(
     data, row_labels, col_labels, ax=None, cbar_kw=None, cbarlabel="", **kwargs
@@ -2424,12 +2403,22 @@ def plot_association_matrix_int(
     panel.Column
         the panel object
     """
+    try:
+        import holoviews as hv
+    except ImportError:
+        raise ImportError(
+            "Holoviews is not installed. Please install it using 'pip install holoviews'."
+        )
+
+    try:
+        import panel as pn
+    except ImportError:
+        raise ImportError(
+            "Panel is not installed. Please install it using 'pip install panel'."
+        )
 
     cmap = cmap if cmap is not None else "coolwarm"
 
-    # if features is None:
-    #     features = list(assoc_mat.columns)
-
     # rename the columns for keeping track of num vs cat columns
     if suffix_dic is not None:
         rename_dic = {c: f"{c}_{suffix_dic[c]}" for c in assoc_mat.columns}
@@ -2453,11 +2442,9 @@ def plot_association_matrix_int(
         fontsize={"title": 12, "ticks": 12, "minor_ticks": 12},
         xrotation=90,
         invert_xaxis=False,
-        invert_yaxis=True,  # title=title_str,
+        invert_yaxis=True,
         xlabel="",
         ylabel="",
-        # gridstyle={'grid_line_color': 'black', 'grid_line_width': 20},
-        # show_grid=True
     )
     title_str = "**Continuous (con) and Categorical (nom) Associations **"
     sub_title_str = (
