@@ -1,13 +1,15 @@
-"""
-This module provides a class to train a base lightgbm and catboost models using early stopping as default.
-The target can be finite discrete (classification) or continuous (regression).
-It can boost from an initial score (aka baseline for catboost), take sample weight as input.
+"""GBM Wrapper
 
-**The module structure is the following:**
+This module offers a class to train base LightGBM and CatBoost models, with early stopping as the default behavior. 
+The target variable can be finite discrete (classification) or continuous (regression). 
+Additionally, the model allows boosting from an initial score (also known as a baseline for CatBoost) and accepts sample weights as input.
 
-- The ``GradientBoosting`` main class to train a lightGBM  or catboost with early stopping
+Module Structure:
+-----------------
+- ``GradientBoosting``: main class to train a lightGBM  or catboost with early stopping
 
 """
+
 from sklearn.model_selection import (
     ShuffleSplit,
     StratifiedShuffleSplit,
@@ -25,47 +27,29 @@ import warnings
 import gc
 import os
 from pathlib import Path
+from arfs.utils import create_dtype_dict
 
-qual_colors_list = [
+QUAL_COLORS = [
     (0.188235, 0.635294, 0.854902),
     (0.898039, 0.682353, 0.219608),
     (0.988235, 0.309804, 0.188235),
     (0.427451, 0.564706, 0.309804),
 ]
+BCKGRND_COLOR = "#f5f5f5"
 
-
-def set_my_plt_style(height=3, width=5, linewidth=2):
-    """This set the style of matplotlib to fivethirtyeight with some modifications (colours, axes)
-
-    Parameters
-    ----------
-    height : float, default = 3
-        fig height in inches (yeah they're still struggling with the metric system) (default ``3``)
-    width : float, default = 5
-        fig width in inches (yeah they're still struggling with the metric system) (default ``5``)
-    linewidth : float
-         (default ``2``)
-
-    """
-    plt.style.use("fivethirtyeight")
-    my_colors_list = qual_colors_list  # Bold_10.hex_colors
-    # myorder = [2, 3, 4, 1, 0, 6, 5, 8, 9, 7]
-    # my_colors_list = [my_colors_list[i] for i in myorder]
-    bckgnd_color = "#f5f5f5"
-    params = {
-        "figure.figsize": (width, height),
-        "axes.prop_cycle": plt.cycler(color=my_colors_list),
-        "axes.facecolor": bckgnd_color,
-        "patch.edgecolor": bckgnd_color,
-        "figure.facecolor": bckgnd_color,
-        "axes.edgecolor": bckgnd_color,
-        "savefig.edgecolor": bckgnd_color,
-        "savefig.facecolor": bckgnd_color,
-        "grid.color": "#d2d2d2",
-        "lines.linewidth": linewidth,
-        "grid.alpha": 0.5,
-    }  # plt.cycler(color=my_colors_list)
-    mpl.rcParams.update(params)
+MPL_PARAMS = {
+    "figure.figsize": (5, 3),
+    "axes.prop_cycle": plt.cycler(color=QUAL_COLORS),
+    "axes.facecolor": BCKGRND_COLOR,
+    "patch.edgecolor": BCKGRND_COLOR,
+    "figure.facecolor": BCKGRND_COLOR,
+    "axes.edgecolor": BCKGRND_COLOR,
+    "savefig.edgecolor": BCKGRND_COLOR,
+    "savefig.facecolor": BCKGRND_COLOR,
+    "grid.color": "#d2d2d2",
+    "lines.linewidth": 2,
+    "grid.alpha": 0.5,
+}
 
 
 class GradientBoosting:
@@ -199,9 +183,9 @@ class GradientBoosting:
             raise KeyError("Provide the objective in the params dict")
 
         if self.cat_feat == "auto":
-            self.cat_feat = list(
-                set(list(X.columns)) - set(list(X.select_dtypes(include=[np.number])))
-            )
+            dtypes_dic = create_dtype_dict(df=X, dic_keys="dtypes")
+            category_cols = dtypes_dic["cat"] + dtypes_dic["time"] + dtypes_dic["unk"]
+            self.cat_feat = category_cols if category_cols else None
 
         if not isinstance(X, (pd.Series, pd.DataFrame)):
             X = pd.DataFrame(X)
@@ -432,7 +416,6 @@ def _fit_early_stopped_lgb(
     )
 
     col_list = list(X.columns)
-    # cat_idx = [col_list.index(c) for c in cat_feat if c in x]
     d_train = lgb.Dataset(
         X_train, label=y_train, categorical_feature=cat_feat, free_raw_data=False
     )
@@ -487,15 +470,6 @@ def _fit_early_stopped_lgb(
     else:
         feval_call = None
 
-    if callable(params["objective"]):
-        fobj_call = params["objective"]
-        params.pop("objective")
-    else:
-        fobj_call = None
-
-    # if 'categorical_feature' in params.keys():
-    #     params.pop('categorical_feature')
-
     watchlist = [d_train, d_valid]
     evals_result = {}
     params["verbosity"] = -1
@@ -510,7 +484,7 @@ def _fit_early_stopped_lgb(
         train_set=d_train,
         valid_sets=watchlist,
         feval=feval_call,
-        fobj=fobj_call,
+        # fobj=fobj_call,
         callbacks=[
             early_stopping(10, verbose=False),
             log_evaluation(verbose_eval),
@@ -519,14 +493,14 @@ def _fit_early_stopped_lgb(
     )
 
     if learning_curve:
-        set_my_plt_style()
-        fig, ax = plt.subplots()
-        ax = lgb.plot_metric(evals_result, ax=ax)
-        up_lim = model.best_iteration + 50
-        ax.axvline(
-            x=model.best_iteration, color="grey", linestyle="--", label="best_iter"
-        )
-        ax.set_xlim([0, up_lim])
+        with mpl.rc_context(MPL_PARAMS):
+            fig, ax = plt.subplots()
+            ax = lgb.plot_metric(evals_result, ax=ax)
+            up_lim = model.best_iteration + 50
+            ax.axvline(
+                x=model.best_iteration, color="grey", linestyle="--", label="best_iter"
+            )
+            ax.set_xlim([0, up_lim])
 
         del d_train
         del d_valid
