@@ -1,41 +1,47 @@
 """
 This module provides preprocessing classes
 
-**The module structure is the following:**
-
-- The ``OrdinalEncoderPandas`` main class for ordinal encoding, takes in a DF and returns a DF of the same shape
-- The ``dtype_column_selector`` for standardizing selection of columns based on their dtypes
-- The ``TreeDiscretizer`` for discretizing continuous columns and auto-group levels of categorical columns
-
+Module Structure:
+-----------------
+- ``OrdinalEncoderPandas``: main class for ordinal encoding, takes in a DF and returns a DF of the same shape
+- ``dtype_column_selector``: for standardizing selection of columns based on their dtypes
+- ``TreeDiscretizer``: class for discretizing continuous columns and auto-group levels of categorical columns
+- ``IntervalToMidpoint``: class for converting pandas numerical intervals into their float midpoint
+- ``PatsyTransformer``: class for encoding data for (generalized) linear models, leveraging Patsy
 """
 
 # Settings and libraries
 from __future__ import print_function
-from tqdm import tqdm
+from tqdm.auto import tqdm
 
 # pandas
 import pandas as pd
+from pandas.api.types import IntervalDtype
 
 # numpy
 import numpy as np
 
+# regular expression
+import re
+
 # sklearn
 from sklearn.preprocessing import OrdinalEncoder
 from sklearn.base import BaseEstimator, TransformerMixin
 
+# patsy
+from patsy import dmatrix, EvalEnvironment, ModelDesc, INTERCEPT
+
+# typing
+from typing import Any, Callable, Union, List, Tuple, Optional, Dict
+
 # ARFS
 from .gbm import GradientBoosting
+from .utils import create_dtype_dict
 
 
 # fix random seed for reproducibility
 np.random.seed(7)
 
-__all__ = [
-    "OrdinalEncoderPandas",
-    "dtype_column_selector",
-    "TreeDiscretizer",
-]
-
 
 class OrdinalEncoderPandas(OrdinalEncoder):
     # class OrdinalEncoderPandas(BaseEstimator, TransformerMixin):
@@ -108,7 +114,6 @@ class OrdinalEncoderPandas(OrdinalEncoder):
         encoded_missing_value=np.nan,
         return_pandas_categorical=False,
     ):
-
         self.dtype_include = dtype_include
         self.dtype_exclude = dtype_exclude
         self.pattern = pattern
@@ -176,7 +181,6 @@ class OrdinalEncoderPandas(OrdinalEncoder):
         X_trans[self.categorical_features_] = super(
             OrdinalEncoderPandas, self
         ).transform(X_trans[self.categorical_features_])
-        # X_trans.loc[:, self.feature_names_in_] = X_trans.loc[:, self.feature_names_in_].apply(pd.to_numeric, errors="ignore", axis=1)
 
         if self.return_pandas_categorical:
             X_trans[self.categorical_features_] = X_trans[
@@ -388,63 +392,71 @@ def cat_var(data, col_excl=None, return_cat=True):
 
 
 class TreeDiscretizer(BaseEstimator, TransformerMixin):
-    """Bin continuous data into intervals and return a pandas DF. It supports regression and binary classification.
-    Discretization (otherwise known as quantization or binning) provides a way to partition continuous features into discrete values.
-    Certain datasets with continuous features may benefit from discretization, because discretization can transform the dataset
-    of continuous attributes to one with only nominal attributes.
-    One-hot encoded discretized features can make a model more expressive, while maintaining interpretability.
-    For instance, pre-processing with a discretizer can introduce nonlinearity to linear models.
-    For more advanced possibilities, in particular smooth ones, see Generating polynomial features further below.
+    """
+    Discretize continuous and/or categorical data using univariate regularized trees, returning a pandas DataFrame.
+    The TreeDiscretizer is designed to support regression and binary classification tasks.
+    Discretization, also known as quantization or binning, allows for the partitioning of continuous features into discrete values.
+    In certain datasets with continuous attributes, discretization can be beneficial as it transforms the dataset into one with only nominal attributes.
+    Additionally, for categorical predictors, grouping levels can help reduce overfitting and create meaningful clusters.
 
-    TreeDiscretizer uses univariate regularized trees (one per column to bin), find the optimal partition and returns
-    pandas numerical interval for numerical continuous columns and pd.Categorical for categorical columns.
-    Similar levels are therefore grouped together. This reduces the dimensionality and regularizes the model.
-    This is a substitution to proper regularization scheme such as
-     - GroupLasso (categorical predictors, which are usually encoded as multiple dummy variables: one for each category.
-                   It makes sense in many analyses to consider these dummy variables (representing one categorical predictor)
-                   together rather than separately)
-     - FusedLasso (One drawback of the Lasso is it ignores ordering of the features, FusedLasso takes into account the ordering)
+    By encoding discretized features, a model can become more expressive while maintaining interpretability.
+    For example, preprocessing with a discretizer can introduce nonlinearity to linear models.
+    For more advanced possibilities, particularly smooth ones, you can refer to the section on generating polynomial features.
+    The TreeDiscretizer function utilizes univariate regularized trees, with one tree per column to be binned.
+    It finds the optimal partition and returns numerical intervals for numerical continuous columns and pd.Categorical for categorical columns.
+    This approach groups similar levels together, reducing dimensionality and regularizing the model.
 
-    TreeDiscretizer handles missing values, both for numerical and categorical predictors. No need for encoding the categorical predictors.
+    TreeDiscretizer handles missing values for both numerical and categorical predictors,
+    eliminating the need for encoding categorical predictors separately.
 
+    Notes
+    -----
+    This is a substitution to proper regularization schemes such as:
+    - GroupLasso: Categorical predictors, which are usually encoded as multiple dummy variables,
+                  are considered together rather than separately.
+    - FusedLasso: Takes into account the ordering of the features.
 
     Parameters
     ----------
-    bin_features : List of string, str or None
-        the list of names of the variable that has to be binned, or "all", "numerical" or "categorical"
+    bin_features : List of string or None
+        The list of names of the variable that has to be binned, or "all", "numerical" or "categorical"
         for splitting and grouping all, only numerical or only categorical columns.
     n_bins : int
-        the number of bins that has to be created while binning the variables in "bin_features" list
-    boost_params : dic
-        the boosting parameters dictionary
+        The number of bins that has to be created while binning the variables in the "bin_features" list.
+    n_bins_max : int, optional
+        The maximum number of levels that a categorical column can have to avoid being binned.
+    num_bins_as_category: bool, default=False
+        Save the numeric bins as pandas category or as pandas interval.
+    boost_params : dict
+        The boosting parameters dictionary.
     raw : bool
-        returns raw levels (non human-interpretable) or levels matching the orginal ones
+        Returns raw levels (non-human-interpretable) or levels matching the original ones.
     task : str
-        either regression or classification (binary)
+        Either regression or classification (binary).
 
     Attributes
     ----------
-    tree_dic : dic
-        the dictionary keys are binned column names and items are the univariate trees
-    bin_upper_bound_dic : dic
-        the upper bound of the numerical intervals
-    cat_bin_dict : dic
-        the mapping dictionary for the categorical columns
-    tree_imputer : dic
-        the missing values are split by the tree and lead to similar splits and are mapped to this value
-    ordinal_encoder_dic : dic
-        dictionary with the fitted encoder, if any
+    tree_dic : dict
+        The dictionary keys are binned column names and items are the univariate trees.
+    bin_upper_bound_dic : dict
+        The upper bound of the numerical intervals.
+    cat_bin_dict : dict
+        The mapping dictionary for the categorical columns.
+    tree_imputer : dict
+        The missing values are split by the tree and lead to similar splits and are mapped to this value.
+    ordinal_encoder_dic : dict
+        Dictionary with the fitted encoder, if any.
     cat_features : list
-        names of the found categorical columns
+        Names of the found categorical columns.
 
     Methods
     -------
     fit(X, y, sample_weight=None)
-        fit the transformer object on data
-    transform(x)
-        apply the fitted transformer object on new data
-    fit_transform(x)
-        fit and apply the transformer object on data
+        Fit the transformer object on data.
+    transform(X)
+        Apply the fitted transformer object on new data.
+    fit_transform(X)
+        Fit and apply the transformer object on data.
 
     Example
     -------
@@ -457,16 +469,19 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
         self,
         bin_features="all",
         n_bins=10,
+        n_bins_max=None,
+        num_bins_as_category=False,
         boost_params=None,
         raw=False,
         task="regression",
     ):
-
         if (boost_params is not None) & (not isinstance(boost_params, dict)):
             raise TypeError("boost_kwargs should be a dictionary")
 
         self.bin_features = bin_features
         self.n_bins = n_bins
+        self.n_bins_max = n_bins_max
+        self.num_bins_as_category = num_bins_as_category
         self.boost_params = {}
         self.raw = raw
         self.task = task
@@ -489,23 +504,31 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
         self.cat_features = None
 
     def fit(self, X, y, sample_weight=None):
-        """Fit the discretizer on `X`.
+        """
+        Fit the TreeDiscretizer on the input data.
+
         Parameters
         ----------
-        X :
-            Input data shape (n_samples, n_features), where `n_samples` is the number of samples and
-            `n_features` is the number of features.
-        y :
-            target for internally fitting the tree(s)
-        sample_weight :
-            sample weight (e.g. exposure) if any
+        X : array-like of shape (n_samples, n_features)
+            The predictor dataframe.
+        y : array-like of shape (n_samples,)
+            The target vector.
+        sample_weight : array-like of shape (n_samples,), optional
+            The weight vector, by default None.
 
         Returns
         -------
-        X :
-            pd.DataFrame with the binned and grouped columns
+        self : object
+            Returns self.
         """
         X = X.copy()
+
+        if not isinstance(X, pd.DataFrame):
+            X = pd.DataFrame(X)
+            X.columns = [f"pred_{i}" for i in range(X.shape[1])]
+
+        self.feature_names_in_ = X.columns.to_numpy()
+
         if self.bin_features is None:
             self.bin_features = list(X.select_dtypes("number").columns)
             self.cat_features = []
@@ -527,13 +550,23 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
             self.bin_features = list(X.select_dtypes(["category", "object"]).columns)
             self.cat_features = self.bin_features
 
+        self.n_unique_table_ = X[self.bin_features].nunique()
+        # transform only the columns with more than n_bins_max
+        self.bin_features = (
+            self.n_unique_table_[self.n_unique_table_ > self.n_bins_max].index.to_list()
+            if self.n_bins_max
+            else self.bin_features
+        )
+
         for col in self.bin_features:
-            if (self.cat_features is not None) and (col in self.cat_features):
+            is_categorical = (self.cat_features is not None) and (
+                col in self.cat_features
+            )
+            if is_categorical:
                 encoder = OrdinalEncoder(
                     handle_unknown="use_encoded_value", unknown_value=np.nan
                 )
                 # create a category for missing
-                # X[col] = X[col].astype("category").cat.add_categories("missing").fillna("missing")
                 X[col] = (
                     X[col]
                     .astype("category")
@@ -542,15 +575,19 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
                 )
                 # encode
                 self.ordinal_encoder_dic[col] = encoder.fit(X[[col]])
-                X[col] = encoder.transform(X[[col]]).ravel()
+                dum = encoder.transform(X[[col]])
+                if isinstance(dum, pd.DataFrame):
+                    X[col] = dum.values.ravel()
+                else:
+                    X[col] = dum.ravel()
+
+            else:
+                encoder = None
 
             gbm_param = self.boost_params.copy()
             tree = GradientBoosting(
-                cat_feat=None,
-                params=gbm_param,
-                show_learning_curve=False,
+                cat_feat=None, params=gbm_param, show_learning_curve=False
             )
-
             tree.fit(X[[col]], y, sample_weight=sample_weight)
 
             # store each fitted tree in a dictionary
@@ -561,23 +598,20 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
             # create monotonicly increasing bin for pd.cut
             X[f"{col}_g"] = tree.predict(X[[col]])
 
-            if (self.cat_features is not None) and (col in self.cat_features):
+            if is_categorical:
+                # retrieve original values
+                dum = encoder.inverse_transform(X[[col]])
+                if isinstance(dum, pd.DataFrame):
+                    X[col] = dum.values.ravel()
+                else:
+                    X[col] = dum.ravel()
+
                 self.cat_bin_dict[col] = (
                     X[[f"{col}_g", col]]
                     .groupby(f"{col}_g")
-                    .aggregate(
-                        lambda x: " / ".join(
-                            map(
-                                str,
-                                encoder.inverse_transform(
-                                    np.expand_dims(X.loc[x.index, col].unique(), axis=1)
-                                ).ravel(),
-                            )
-                        )
-                    )
-                    .to_dict()[col]
+                    .apply(lambda x: " / ".join(map(str, x[col].unique())))
+                    .to_dict()
                 )
-
             else:
                 bin_array = (
                     X[[f"{col}_g", col]]
@@ -604,23 +638,26 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
                 ]
 
             del tree
+
         return self
 
     def transform(self, X):
-        """Apply the discretizer on `X`.
+        """
+        Apply the discretizer on `X`. Only the columns with more than n_bins_max unique values will be transformed.
+
         Parameters
         ----------
-        X :
-            Input data shape (n_samples, n_features), where `n_samples` is the number of samples and
+        X : array-like of shape (n_samples, n_features)
+            Input data with shape (n_samples, n_features), where `n_samples` is the number of samples and
             `n_features` is the number of features.
 
-
         Returns
         -------
-        X :
-            pd.DataFrame with the binned and grouped columns
+        X : pd.DataFrame
+            DataFrame with the binned and grouped columns.
         """
         X = X.copy()
+
         for col in self.bin_features:
             if self.raw:
                 # predict each univariate tree
@@ -647,13 +684,17 @@ class TreeDiscretizer(BaseEstimator, TransformerMixin):
                         X[col],
                         bins=self.bin_upper_bound_dic[col],
                         include_lowest=True,
-                        precision=1,
+                        precision=2,
                     )
+
+                    if not self.num_bins_as_category:
+                        X[col] = X[col].astype(IntervalDtype())
         return X
 
 
 def highlight_discarded(s):
-    """highlight X in red and V in green.
+    """
+    highlight X in red and V in green.
 
     Parameters
     ----------
@@ -670,48 +711,309 @@ def highlight_discarded(s):
     ]
 
 
-def make_fs_summary(selector_pipe):
-    """make_fs_summary makes a summary dataframe highlighting at which step a
-    given predictor has been rejected (if any).
+class IntervalToMidpoint(BaseEstimator, TransformerMixin):
+    """
+    IntervalToMidpoint is a transformer that converts numerical intervals in a pandas DataFrame to their midpoints.
 
     Parameters
     ----------
-    selector_pipe : sklearn.pipeline.Pipeline
-        the feature selector pipeline.
+    cols : list of str or str, default "all"
+        The column(s) to transform. If "all", all columns with numerical intervals will be transformed.
 
-    Examples
-    --------
-    >>> groot_pipeline = Pipeline([
-    ... ('missing', MissingValueThreshold()),
-    ... ('unique', UniqueValuesThreshold()),
-    ... ('cardinality', CardinalityThreshold()),
-    ... ('collinearity', CollinearityThreshold(threshold=0.5)),
-    ... ('lowimp', VariableImportance(eval_metric='poisson', objective='poisson', verbose=2)),
-    ... ('grootcv', GrootCV(objective='poisson', cutoff=1, n_folds=3, n_iter=5))])
-    >>> groot_pipeline.fit_transform(
-        X=df[predictors],
-        y=df[target],
-        lowimp__sample_weight=df[weight],
-        grootcv__sample_weight=df[weight])
-    >>> fs_summary_df = make_fs_summary(groot_pipeline)
+    Attributes
+    ----------
+    cols : list of str or str
+        The column(s) to transform.
+    float_interval_cols_ : list of str
+        The columns with numerical interval data types in the input DataFrame.
+    columns_to_transform_ : list of str
+        The columns to be transformed based on the specified `cols` attribute.
+
+    Methods
+    -------
+    fit(X, y=None)
+        Fit the transformer on the input data.
+    transform(X)
+        Transform the input data by converting numerical intervals to midpoints.
+    inverse_transform(X)
+        Inverse transform is not implemented for this transformer.
     """
 
-    tag_df = pd.DataFrame({"predictor": selector_pipe[0].feature_names_in_})
-    for selector_name in selector_pipe.named_steps.keys():
-        if hasattr(selector_pipe.named_steps[selector_name], "feature_names_in_"):
-            feature_in = selector_pipe.named_steps[selector_name].feature_names_in_
-            to_drop = list(
-                set(selector_pipe.named_steps[selector_name].feature_names_in_)
-                - set(selector_pipe.named_steps[selector_name].get_feature_names_out())
-            )
-            tag_df[selector_name] = np.where(
-                tag_df["predictor"].isin(to_drop), 0, 1
-            ) * np.where(tag_df["predictor"].isin(feature_in), 1, np.nan)
+    def __init__(self, cols: Union[List[str], str] = "all"):
+        self.cols = cols
 
-    col_to_apply_style = tag_df.columns[1:]
-    tag_df = (
-        tag_df.style.apply(highlight_discarded, subset=col_to_apply_style)
-        .applymap(lambda x: "" if x == x else "background-color: #ffa500")
-        .format(precision=0)
+    def fit(self, X: pd.DataFrame = None, y: pd.Series = None):
+        """
+        Fit the transformer on the input data.
+
+        Parameters
+        ----------
+        X :
+            The input data to fit the transformer on.
+        y :
+            Ignored parameter.
+
+        Returns
+        -------
+        self : IntervalToMidpoint
+            The fitted transformer object.
+        """
+        data = X.copy()
+
+        if self.cols == "all":
+            self.cols = data.columns
+
+        self.float_interval_cols_ = create_dtype_dict(X, dic_keys="dtypes")[
+            "num_interval"
+        ]
+        self.columns_to_transform_ = list(
+            set(self.cols).intersection(set(self.float_interval_cols_))
+        )
+        return self
+
+    def transform(self, X: pd.DataFrame):
+        """
+        Transform the input data by converting numerical intervals to midpoints.
+
+        Parameters
+        ----------
+        X : pd.DataFrame
+            The input data to transform.
+
+        Returns
+        -------
+        X : pd.DataFrame
+            The transformed data with numerical intervals replaced by their midpoints.
+        """
+        X = X.copy()
+        for c in self.columns_to_transform_:
+            X.loc[:, c] = find_interval_midpoint(X[c])
+            X.loc[:, c] = X[c].astype(float)
+        return X
+
+    def inverse_transform(self, X: pd.DataFrame):
+        """
+        Inverse transform is not implemented for this transformer.
+
+        Parameters
+        ----------
+        X : pd.DataFrame
+            The input data to perform inverse transform on.
+
+        Raises
+        ------
+        NotImplementedError
+            Raised since inverse transform is not implemented for this transformer.
+        """
+        raise NotImplementedError(
+            "inverse_transform is not implemented for this transformer."
+        )
+
+
+def transform_interval_to_midpoint(
+    X: pd.DataFrame, cols: Union[List[str], str] = "all"
+) -> pd.DataFrame:
+    """
+    Transforms interval columns in a pandas DataFrame to their midpoint values.
+
+    Notes
+    -----
+    Equivalent function to ``IntervalToMidpoint`` without the estimator API
+
+    Parameters
+    ----------
+    X : pd.DataFrame
+        The input DataFrame containing the data to be transformed.
+    cols : list of str or str
+        The columns to be transformed. Defaults to "all" which transforms all columns.
+
+    Returns
+    -------
+    pd.DataFrame :
+        The transformed DataFrame with interval columns replaced by their midpoint values.
+
+    Raises
+    ------
+    TypeError :
+        If the input data is not a pandas DataFrame.
+    """
+    if cols == "all":
+        cols = X.columns
+
+    X = X.copy()
+    float_interval_cols_ = create_dtype_dict(X, dic_keys="dtypes")["num_interval"]
+    columns_to_transform_ = list(set(cols).intersection(set(float_interval_cols_)))
+    for c in columns_to_transform_:
+        X.loc[:, c] = find_interval_midpoint(X[c])
+    return X
+
+
+def find_interval_midpoint(interval_series: pd.Series) -> np.ndarray:
+    """Find the midpoint (or left/right bound if the interval contains Inf).
+
+    Parameters
+    ----------
+    interval_series : pd.Series
+        series of pandas intervals.
+
+    Returns
+    -------
+    np.ndarray
+        Array of midpoints or bounds of the intervals.
+    """
+    left = interval_series.array.left
+    right = interval_series.array.right
+    mid = interval_series.array.mid
+    left_inf = np.isinf(left)
+    right_inf = np.isinf(right)
+
+    return np.where(
+        left_inf & right_inf,
+        np.inf,
+        np.where(left_inf, right, np.where(right_inf, left, mid)),
     )
-    return tag_df
+
+
+class PatsyTransformer(BaseEstimator, TransformerMixin):
+    """Transformer using patsy-formulas.
+
+    PatsyTransformer transforms a pandas DataFrame (or dict-like)
+    according to the formula and produces a numpy array.
+
+    Parameters
+    ----------
+    formula : string or formula-like
+        Pasty formula used to transform the data.
+
+    add_intercept : boolean, default=False
+        Wether to add an intersept. By default scikit-learn has built-in
+        intercepts for all models, so we don't add an intercept to the data,
+        even if one is specified in the formula.
+
+    eval_env : environment or int, default=0
+        Envirionment in which to evalute the formula.
+        Defaults to the scope in which PatsyModel was instantiated.
+
+    NA_action : string or NAAction, default="drop"
+        What to do with rows that contain missing values. You can ``"drop"``
+        them, ``"raise"`` an error, or for customization, pass an `NAAction`
+        object.  See ``patsy.NAAction`` for details on what values count as
+        'missing' (and how to alter this).
+
+    Attributes
+    ----------
+    feature_names_ : list of string
+        Column names / keys of training data.
+
+    return_type : string, default="dataframe"
+        data type that transform method will return. Default is ``"dataframe"``
+        for numpy array, but if you would like to get Pandas dataframe (for
+        example for using it in scikit transformers with dataframe as input
+        use ``"dataframe"`` and if numpy array use ``"ndarray"``
+
+    Note
+    ----
+    PastyTransformer does by default not add an intercept, even if you
+    specified it in the formula. You need to set add_intercept=True.
+
+    As scikit-learn transformers can not ouput y, the formula
+    should not contain a left hand side.  If you need to transform both
+    features and targets, use PatsyModel.
+    """
+
+    def __init__(
+        self,
+        formula=None,
+        add_intercept=True,
+        eval_env=0,
+        NA_action="drop",
+        return_type="dataframe",
+    ):
+        self.formula = formula
+        self.eval_env = eval_env
+        self.add_intercept = add_intercept
+        self.NA_action = NA_action
+        self.return_type = return_type
+
+    def fit(self, data, y=None):
+        """Fit the scikit-learn model using the formula.
+
+        Parameters
+        ----------
+        data : dict-like (pandas dataframe)
+            Input data. Column names need to match variables in formula.
+        """
+        self._fit_transform(data, y)
+        return self
+
+    def fit_transform(self, data, y=None):
+        """Fit the scikit-learn model using the formula and transform it.
+
+        Parameters
+        ----------
+        data : dict-like (pandas dataframe)
+            Input data. Column names need to match variables in formula.
+
+        Returns
+        -------
+        X_transform : ndarray
+            Transformed data
+        """
+        return self._fit_transform(data, y)
+
+    def _fit_transform(self, data, y=None):
+        if not isinstance(data, pd.DataFrame):
+            data = pd.DataFrame(data)
+            data.columns = [f"pred_{i}" for i in range(data.shape[1])]
+
+        if not isinstance(y, pd.Series):
+            y = pd.Series(y)
+            y.name = "target"
+
+        target_name = y.name if y is not None else "y"
+        self.formula = self.formula or " + ".join(
+            data.columns.difference([target_name])
+        )
+        eval_env = EvalEnvironment.capture(self.eval_env, reference=2)
+        # self.formula = _drop_intercept(self.formula, self.add_intercept)
+
+        design = dmatrix(
+            self.formula,
+            data,
+            NA_action=self.NA_action,
+            return_type="dataframe",
+            eval_env=eval_env,
+        )
+        self.design_ = design.design_info
+
+        if self.return_type == "dataframe":
+            return design
+        else:
+            return np.array(design)
+
+    def transform(self, data):
+        """Transform with estimator using formula.
+
+        Transform the data using formula, then transform it
+        using the estimator.
+
+        Parameters
+        ----------
+        data : dict-like (pandas dataframe)
+            Input data. Column names need to match variables in formula.
+        """
+        if self.return_type == "dataframe":
+            return dmatrix(self.design_, data, return_type="dataframe")
+        else:
+            return np.array(dmatrix(self.design_, data))
+
+
+def _drop_intercept(formula, add_intercept):
+    """Drop the intercept from formula if not add_intercept"""
+    if not add_intercept:
+        if not isinstance(formula, ModelDesc):
+            formula = ModelDesc.from_formula(formula)
+        if INTERCEPT in formula.rhs_termlist:
+            formula.rhs_termlist.remove(INTERCEPT)
+        return formula
+    return formula
